# Generated from D:/Programming/Projects/Apitax/grammar/src\Ah2.g4 by ANTLR 4.7
from antlr4 import *
if __name__ is not None and "." in __name__:
    from .Ah2Parser import Ah2Parser
else:
    from Ah2Parser import Ah2Parser

# This class defines a complete listener for a parse tree produced by Ah2Parser.
class Ah2Listener(ParseTreeListener):

    # Enter a parse tree produced by Ah2Parser#prog.
    def enterProg(self, ctx:Ah2Parser.ProgContext):
        pass

    # Exit a parse tree produced by Ah2Parser#prog.
    def exitProg(self, ctx:Ah2Parser.ProgContext):
        pass


    # Enter a parse tree produced by Ah2Parser#statements.
    def enterStatements(self, ctx:Ah2Parser.StatementsContext):
        pass

    # Exit a parse tree produced by Ah2Parser#statements.
    def exitStatements(self, ctx:Ah2Parser.StatementsContext):
        pass


    # Enter a parse tree produced by Ah2Parser#statement.
    def enterStatement(self, ctx:Ah2Parser.StatementContext):
        pass

    # Exit a parse tree produced by Ah2Parser#statement.
    def exitStatement(self, ctx:Ah2Parser.StatementContext):
        pass


    # Enter a parse tree produced by Ah2Parser#expr.
    def enterExpr(self, ctx:Ah2Parser.ExprContext):
        pass

    # Exit a parse tree produced by Ah2Parser#expr.
    def exitExpr(self, ctx:Ah2Parser.ExprContext):
        pass


    # Enter a parse tree produced by Ah2Parser#set_var.
    def enterSet_var(self, ctx:Ah2Parser.Set_varContext):
        pass

    # Exit a parse tree produced by Ah2Parser#set_var.
    def exitSet_var(self, ctx:Ah2Parser.Set_varContext):
        pass


    # Enter a parse tree produced by Ah2Parser#scoping.
    def enterScoping(self, ctx:Ah2Parser.ScopingContext):
        pass

    # Exit a parse tree produced by Ah2Parser#scoping.
    def exitScoping(self, ctx:Ah2Parser.ScopingContext):
        pass


    # Enter a parse tree produced by Ah2Parser#name.
    def enterName(self, ctx:Ah2Parser.NameContext):
        pass

    # Exit a parse tree produced by Ah2Parser#name.
    def exitName(self, ctx:Ah2Parser.NameContext):
        pass


    # Enter a parse tree produced by Ah2Parser#exports.
    def enterExports(self, ctx:Ah2Parser.ExportsContext):
        pass

    # Exit a parse tree produced by Ah2Parser#exports.
    def exitExports(self, ctx:Ah2Parser.ExportsContext):
        pass


    # Enter a parse tree produced by Ah2Parser#imports.
    def enterImports(self, ctx:Ah2Parser.ImportsContext):
        pass

    # Exit a parse tree produced by Ah2Parser#imports.
    def exitImports(self, ctx:Ah2Parser.ImportsContext):
        pass


    # Enter a parse tree produced by Ah2Parser#execute.
    def enterExecute(self, ctx:Ah2Parser.ExecuteContext):
        pass

    # Exit a parse tree produced by Ah2Parser#execute.
    def exitExecute(self, ctx:Ah2Parser.ExecuteContext):
        pass


    # Enter a parse tree produced by Ah2Parser#inject.
    def enterInject(self, ctx:Ah2Parser.InjectContext):
        pass

    # Exit a parse tree produced by Ah2Parser#inject.
    def exitInject(self, ctx:Ah2Parser.InjectContext):
        pass


    # Enter a parse tree produced by Ah2Parser#variable_types.
    def enterVariable_types(self, ctx:Ah2Parser.Variable_typesContext):
        pass

    # Exit a parse tree produced by Ah2Parser#variable_types.
    def exitVariable_types(self, ctx:Ah2Parser.Variable_typesContext):
        pass


